import {
  FieldHighlighterProvider,
  JaenPage,
  PageConfig,
  useAuth,
  withAuthSecurity
} from '@atsnek/jaen'
import {Flex} from '@chakra-ui/react'
import {GatsbyBrowser, PageProps, Slice} from 'gatsby'
import React, {useEffect, useMemo} from 'react'
import * as Sentry from '@sentry/gatsby'

import {theme} from '../theme/jaen-theme/index'
import {DynamicPageRenderer} from './DynamicPageRenderer'
import Layout from './Layout'

// Import other necessary components here

interface PageContext {
  pageConfig?: PageConfig
  jaenPageId?: string
}

interface CustomPageElementProps
  extends PageProps<
    {
      jaenPage?: JaenPage
      allJaenPage?: {
        nodes: Array<JaenPage>
      }
    },
    PageContext
  > {
  element: React.ReactElement<any, string | React.JSXElementConstructor<any>>
}

const CustomPageElement: React.FC<CustomPageElementProps> = ({
  element,
  ...props
}) => {
  const auth = useAuth()

  if (auth.isAuthenticated && auth.user) {
    Sentry.setUser({
      email: auth.user.profile.email,
      id: auth.user.profile.sub,
      username: auth.user.profile.preferred_username,
      details: auth
    })
  }

  const withoutJaenFrame = props.pageContext?.pageConfig?.withoutJaenFrame

  if (!withoutJaenFrame) {
    return (
      <Flex
        pos="relative"
        flexDirection="column"
        visibility={
          props.pageContext?.pageConfig?.auth?.isRequired &&
          !auth.isAuthenticated
            ? 'hidden'
            : 'visible'
        }>
        {auth.isAuthenticated && (
          <Slice
            alias="jaen-frame"
            jaenPageId={props.pageContext?.jaenPageId}
            pageConfig={props.pageContext?.pageConfig as any}
          />
        )}

        <Layout pageProps={props}>{element}</Layout>
      </Flex>
    )
  }

  return <Layout pageProps={props}>{element}</Layout>
}

export interface WithJaenPageProviderProps
  extends PageProps<
    {
      jaenPage?: JaenPage
      allJaenPage?: {
        nodes: Array<JaenPage>
      }
    },
    PageContext
  > {}

const SecureRendered = withAuthSecurity(DynamicPageRenderer)

const withJaenPageProvider = <P extends WithJaenPageProviderProps>(
  Component: React.ComponentType<P>
): React.FC<P> => {
  return props => {
    return <SecureRendered {...props} Component={Component} />
  }
}

const JaenPageElement = withJaenPageProvider(CustomPageElement)

export const wrapPageElement: GatsbyBrowser['wrapPageElement'] = ({
  element,
  props
}) => {
  return (
    <FieldHighlighterProvider path={props.location.pathname} theme={theme}>
      <JaenPageElement element={element} {...props} />
    </FieldHighlighterProvider>
  )
}

export interface UseTemplateReturn {}
